@@ -1,11 +1,12 @@
-# {{{ Version, package, new, etc
 
 package DBIx::SearchBuilder;
 
 use strict;
-use vars qw($VERSION);
 
-$VERSION = "1.30_03";
+use vars qw($VERSION);
+$VERSION = "1.37_01";
+
+use Clone qw();
 
 =head1 NAME
 
@@ -59,7 +60,7 @@ You can try to override just about every other method here, as long as you think
 are doing.
 
 =head1 METHOD NAMING
- 
+
 Each method has a lower case alias; '_' is used to separate words.
 For example, the method C<RedoSearch> has the alias C<redo_search>.
 
@@ -67,7 +68,6 @@ For example, the method C<RedoSearch> has the alias C<redo_search>.
 
 =cut
 
-# {{{ sub new
 
 =head2 new
 
@@ -93,9 +93,7 @@ sub new {
     return ($self);
 }
 
-# }}}
 
-# {{{ sub _Init
 
 =head2 _Init
 
@@ -114,9 +112,7 @@ sub _Init {
     $self->CleanSlate();
 }
 
-# }}}
 
-# {{{ sub CleanSlate
 
 =head2 CleanSlate
 
@@ -155,12 +151,52 @@ sub CleanSlate {
 
     #we have no limit statements. DoSearch won't work.
     $self->_isLimited(0);
-
 }
 
-# }}}
+=head2 Clone
+
+Returns copy of the current object with all search restrictions.
+
+=cut
+
+sub Clone
+{
+    my $self = shift;
+
+    my $obj = bless {}, ref($self);
+    %$obj = %$self;
+
+    delete $obj->{$_} for qw(
+        items
+    );
+    $obj->{'must_redo_search'} = 1;
+    $obj->{'itemscount'}       = 0;
+    
+    $obj->{$_} = Clone::clone($obj->{$_}) for ( $self->_ClonedAttributes );
+    return $obj;
+}
+
+=head2 _ClonedAttributes
+
+Returns list of the object's fields that should be copied.
+
+If your subclass store references in the object that should be copied while
+clonning then you probably want override this method and add own values to
+the list.
+
+=cut
+
+sub _ClonedAttributes
+{
+    return qw(
+        aliases
+        left_joins
+        subclauses
+        restrictions
+    );
+}
+
 
-# {{{ sub _Handle
 
 =head2 _Handle  [DBH]
 
@@ -176,10 +212,6 @@ sub _Handle {
     return ( $self->{'DBIxHandle'} );
 }
 
-# }}}
-
-# {{{ sub _DoSearch
-    
 =head2 _DoSearch
 
 This internal private method actually executes the search on the database;
@@ -211,7 +243,6 @@ sub _DoSearch {
     return $self->_RecordCount;
 }
 
-# }}}
 
 =head2 AddRecord RECORD
 
@@ -239,7 +270,6 @@ sub _RecordCount {
 }
 
 
-# {{{ sub _DoCount
 
 =head2 _DoCount
 
@@ -265,7 +295,6 @@ sub _DoCount {
     return ( $row[0] );
 }
 
-# }}}
 
 
 =head2 _ApplyLimits STATEMENTREF
@@ -286,40 +315,24 @@ sub _ApplyLimits {
     $self->_Handle->ApplyLimits($statementref, $self->RowsPerPage, $self->FirstRow);
     $$statementref =~ s/main\.\*/join(', ', @{$self->{columns}})/eg
 	    if $self->{columns} and @{$self->{columns}};
-    if (my $groupby = $self->_GroupClause) {
-	    $$statementref =~ s/(LIMIT \d+)?$/$groupby $1/;
-    }
-    
 }
 
-# {{{ sub _DistinctQuery
 
 =head2 _DistinctQuery STATEMENTREF
 
 This routine takes a reference to a scalar containing an SQL statement. 
 It massages the statement to ensure a distinct result set is returned.
 
-
 =cut
 
 sub _DistinctQuery {
     my $self = shift;
     my $statementref = shift;
-    my $table = shift;
 
     # XXX - Postgres gets unhappy with distinct and OrderBy aliases
-    if (exists $self->{'order_clause'} && $self->{'order_clause'} =~ /(?<!main)\./) {
-        $$statementref = "SELECT main.* FROM $$statementref";
-    }
-    else {
-	$self->_Handle->DistinctQuery($statementref, $table)
-    }
+    $self->_Handle->DistinctQuery($statementref, $self)
 }
 
-# }}}
-
-# {{{ sub _BuildJoins
-
 =head2 _BuildJoins
 
 Build up all of the joins we need to perform this query.
@@ -334,8 +347,6 @@ sub _BuildJoins {
 
 }
 
-# }}}
-# {{{ sub _isJoined
 
 =head2 _isJoined 
 
@@ -353,10 +364,8 @@ sub _isJoined {
 
 }
 
-# }}}
 
 
-# {{{ sub _LimitClause
 
 # LIMIT clauses are used for restricting ourselves to subsets of the search.
 
@@ -379,9 +388,7 @@ sub _LimitClause {
     return $limit_clause;
 }
 
-# }}}
 
-# {{{ sub _isLimited
 
 =head2 _isLimited
 
@@ -399,11 +406,8 @@ sub _isLimited {
     }
 }
 
-# }}}
 
-# }}} Private utility methods
 
-# {{{ BuildSelectQuery
 
 =head2 BuildSelectQuery
 
@@ -422,22 +426,20 @@ sub BuildSelectQuery {
 
     # DISTINCT query only required for multi-table selects
     if ($self->_isJoined) {
-        $self->_DistinctQuery(\$QueryString, $self->Table);
+        $self->_DistinctQuery(\$QueryString);
     } else {
         $QueryString = "SELECT main.* FROM $QueryString";
+        $QueryString .= $self->_GroupClause;
+        $QueryString .= $self->_OrderClause;
     }
 
-    $QueryString .= $self->_OrderClause;
-
     $self->_ApplyLimits(\$QueryString);
 
     return($QueryString)
 
 }
 
-# }}}
 
-# {{{ BuildSelectCountQuery
 
 =head2 BuildSelectCountQuery
 
@@ -467,11 +469,8 @@ sub BuildSelectCountQuery {
     return ($QueryString);
 }
 
-# }}}
 
-# {{{ Methods dealing traversing rows within the found set
 
-# {{{ sub Next
 
 =head2 Next
 
@@ -502,9 +501,7 @@ sub Next {
     }
 }
 
-# }}}
 
-# {{{ sub GotoFirstItem
 
 =head2 GotoFirstItem
 
@@ -520,9 +517,7 @@ sub GotoFirstItem {
     $self->GotoItem(0);
 }
 
-# }}}
 
-# {{{ sub GotoItem
 
 
 =head2 GotoItem
@@ -539,9 +534,7 @@ sub GotoItem {
     $self->{'itemscount'} = $item;
 }
 
-# }}}
 
-# {{{ sub First
 
 =head2 First
 
@@ -555,9 +548,7 @@ sub First {
     return ( $self->Next );
 }
 
-# }}}
 
-# {{{ sub Last
 
 =head2 Last
 
@@ -571,9 +562,7 @@ sub Last {
     return ( $self->Next );
 }
 
-# }}}
 
-# {{{ ItemsArrayRef
 
 =head2 ItemsArrayRef
 
@@ -595,11 +584,8 @@ sub ItemsArrayRef {
     return ( $self->{'items'} || [] );
 }
 
-# }}}
 
-# }}}
 
-# {{{ sub NewItem
 
 =head2 NewItem
 
@@ -615,9 +601,7 @@ sub NewItem {
 "DBIx::SearchBuilder needs to be subclassed. you can't use it directly.\n";
 }
 
-# }}}
 
-# {{{ sub RedoSearch
 
 =head2 RedoSearch
 
@@ -631,11 +615,8 @@ sub RedoSearch {
     $self->{'must_redo_search'} = 1;
 }
 
-# }}}
 
-# {{{ Routines dealing with Restrictions (where subclauses)
 
-# {{{ sub UnLimit
 
 =head2 UnLimit
 
@@ -649,9 +630,7 @@ sub UnLimit {
     $self->_isLimited(-1);
 }
 
-# }}}
 
-# {{{ sub Limit
 
 =head2 Limit
 
@@ -702,6 +681,11 @@ STARTSWITH is like LIKE, except it only appends a % at the end of the string
 
 ENDSWITH is like LIKE, except it prepends a % to the beginning of the string
 
+=item "MATCHES"
+
+MATCHES is equivalent to the database's LIKE -- that is, it's actually LIKE, but
+doesn't surround the string in % signs as LIKE does.
+
 =back
 
 =item ENTRYAGGREGATOR 
@@ -751,7 +735,9 @@ sub Limit {
         elsif ( $args{'OPERATOR'} =~ /ENDSWITH/i ) {
             $args{'VALUE'}    = "%" . $args{'VALUE'};
             $args{'OPERATOR'} = "LIKE";
-        }
+        } 
+	
+	$args{'OPERATOR'} =~ s/MATCHES/LIKE/i;  # MATCHES becomes LIKE, with no % stuff
 
         #if we're explicitly told not to to quote the value or
         # we're doing an IS or IS NOT (null), don't quote the operator.
@@ -787,9 +773,7 @@ sub Limit {
     }
 }
 
-# }}}
 
-# {{{ sub ShowRestrictions
 
 =head2 ShowRestrictions
 
@@ -807,9 +791,7 @@ sub ShowRestrictions {
 
 }
 
-# }}}
 
-# {{{ sub ImportRestrictions
 
 =head2 ImportRestrictions
 
@@ -825,9 +807,7 @@ sub ImportRestrictions {
     $self->{'where_clause'} = shift;
 }
 
-# }}}
 
-# {{{ sub _GenericRestriction
 
 sub _GenericRestriction {
     my $self = shift;
@@ -946,9 +926,7 @@ sub _GenericRestriction {
 
 }
 
-# }}}
 
-# {{{ Parentheses Control
 sub _OpenParen {
     my ( $self, $clause ) = @_;
     $self->{_open_parens}{$clause}++;
@@ -966,9 +944,7 @@ sub _CloseParen {
     }
 }
 
-# }}}
 
-# {{{ sub _AddRestriction
 sub _AddSubClause {
     my $self      = shift;
     my $clauseid  = shift;
@@ -978,9 +954,7 @@ sub _AddSubClause {
 
 }
 
-# }}}
 
-# {{{ sub _WhereClause
 
 sub _WhereClause {
     my $self = shift;
@@ -1010,9 +984,7 @@ sub _WhereClause {
 
 }
 
-# }}}
 
-# {{{ sub _CompileGenericRestrictions
 
 #Compile the restrictions to a WHERE Clause
 
@@ -1032,31 +1004,26 @@ sub _CompileGenericRestrictions {
     }
 }
 
-# }}}
 
-# }}}
 
-# {{{ Routines dealing with ordering
 
-# {{{ sub OrderBy
 
-=head2 Orderby PARAMHASH
+=head2 OrderBy PARAMHASH
 
-Orders the returned results by ALIAS.FIELD ORDER. (by default 'main.id ASC')
+Orders the returned results by ALIAS.FIELD ORDER.
 
 Takes a paramhash of ALIAS, FIELD and ORDER.  
-ALIAS defaults to main
-FIELD defaults to the primary key of the main table.  Also accepts C<FUNCTION(FIELD)> format
-ORDER defaults to ASC(ending).  DESC(ending) is also a valid value for OrderBy
+ALIAS defaults to C<main>.
+FIELD has no default value.
+ORDER defaults to ASC(ending). DESC(ending) is also a valid value for OrderBy.
 
+FIELD also accepts C<FUNCTION(FIELD)> format.
 
 =cut
 
 sub OrderBy {
     my $self = shift;
-    my %args = ( @_ );
-
-    $self->OrderByCols( \%args );
+    $self->OrderByCols( { @_ } );
 }
 
 =head2 OrderByCols ARRAY
@@ -1069,10 +1036,24 @@ The result set is ordered by the items in the array.
 sub OrderByCols {
     my $self = shift;
     my @args = @_;
-    my $row;
-    my $clause;
 
-    foreach $row ( @args ) {
+    $self->{'order_by'} = \@args;
+    $self->RedoSearch();
+}
+
+=head2 _OrderClause
+
+returns the ORDER BY clause for the search.
+
+=cut
+
+sub _OrderClause {
+    my $self = shift;
+
+    return '' unless $self->{'order_by'};
+
+    my $clause = '';
+    foreach my $row ( @{$self->{'order_by'}} ) {
 
         my %rowhash = ( ALIAS => 'main',
 			FIELD => undef,
@@ -1102,40 +1083,14 @@ sub OrderByCols {
             $clause .= $rowhash{'ORDER'};
         }
     }
+    $clause = " ORDER BY$clause " if $clause;
 
-    if ($clause) {
-	$self->{'order_clause'} = "ORDER BY" . $clause;
-    }
-    else {
-	$self->{'order_clause'} = "";
-    }
-    $self->RedoSearch();
+    return $clause;
 }
 
-# }}} 
 
-# {{{ sub _OrderClause
 
-=head2 _OrderClause
 
-returns the ORDER BY clause for the search.
-
-=cut
-
-sub _OrderClause {
-    my $self = shift;
-
-    return '' unless $self->{'order_clause'};
-    return ($self->{'order_clause'});
-}
-
-# }}}
-
-# }}}
-
-# {{{ Routines dealing with grouping
-
-# {{{ GroupBy (OBSOLETE)
 
 =head2 GroupBy  (DEPRECATED)
 
@@ -1145,9 +1100,7 @@ Alias for the GroupByCols method.
 
 sub GroupBy { (shift)->GroupByCols( @_ ) }
 
-# }}}
 
-# {{{ GroupByCols
 
 =head2 GroupByCols ARRAY_OF_HASHES
 
@@ -1158,10 +1111,26 @@ Each hash contains the keys ALIAS and FIELD. ALIAS defaults to 'main' if ignored
 sub GroupByCols {
     my $self = shift;
     my @args = @_;
+
+    $self->{'group_by'} = \@args;
+    $self->RedoSearch();
+}
+
+
+=head2 _GroupClause
+
+Private function to return the "GROUP BY" clause for this query.
+
+=cut
+
+sub _GroupClause {
+    my $self = shift;
+    return '' unless $self->{'group_by'};
+
     my $row;
     my $clause;
 
-    foreach $row ( @args ) {
+    foreach $row ( @{$self->{'group_by'}} ) {
         my %rowhash = ( ALIAS => 'main',
 			FIELD => undef,
 			%$row
@@ -1181,37 +1150,16 @@ sub GroupByCols {
     }
 
     if ($clause) {
-	$self->{'group_clause'} = "GROUP BY" . $clause;
+	return " GROUP BY" . $clause . " ";
     }
     else {
-	$self->{'group_clause'} = "";
+	return '';
     }
-    $self->RedoSearch();
-}
-# }}} 
-
-# {{{ _GroupClause
-
-=head2 _GroupClause
-
-Private function to return the "GROUP BY" clause for this query.
-
-=cut
-
-sub _GroupClause {
-    my $self = shift;
-
-    return '' unless $self->{'group_clause'};
-    return ($self->{'group_clause'});
 }
 
-# }}}
 
-# }}}
 
-# {{{ routines dealing with table aliases and linking tables
 
-# {{{ sub NewAlias
 
 =head2 NewAlias
 
@@ -1234,9 +1182,7 @@ sub NewAlias {
     return $alias;
 }
 
-# }}}
 
-# {{{ sub _GetAlias
 
 # _GetAlias is a private function which takes an tablename and
 # returns a new alias for that table without adding something
@@ -1254,9 +1200,7 @@ sub _GetAlias {
 
 }
 
-# }}}
 
-# {{{ sub Join
 
 =head2 Join
 
@@ -1294,34 +1238,24 @@ sub Join {
 
 }
 
-# }}}
 
-# }}}
 
-# {{{ Deal with 'pages' of results'
 
-# {{{ sub NextPage
 
 sub NextPage {
     my $self = shift;
     $self->FirstRow( $self->FirstRow + $self->RowsPerPage );
 }
 
-# }}}
 
-# {{{ sub FirstPage
 sub FirstPage {
     my $self = shift;
     $self->FirstRow(1);
 }
 
-# }}}
 
-# {{{ sub LastPage
 
-# }}}
 
-# {{{ sub PrevPage
 
 sub PrevPage {
     my $self = shift;
@@ -1333,9 +1267,7 @@ sub PrevPage {
     }
 }
 
-# }}}
 
-# {{{ sub GotoPage
 
 sub GotoPage {
     my $self = shift;
@@ -1348,9 +1280,7 @@ sub GotoPage {
     }
 }
 
-# }}}
 
-# {{{ sub RowsPerPage
 
 =head2 RowsPerPage
 
@@ -1367,9 +1297,7 @@ sub RowsPerPage {
     return ( $self->{'show_rows'} );
 }
 
-# }}}
 
-# {{{ sub FirstRow
 
 =head2 FirstRow
 
@@ -1395,13 +1323,9 @@ sub FirstRow {
     return ( $self->{'first_row'} );
 }
 
-# }}}
 
-# }}}
 
-# {{{ Public utility methods
 
-# {{{ sub _ItemsCounter
 
 =head2 _ItemsCounter
 
@@ -1414,9 +1338,7 @@ sub _ItemsCounter {
     return $self->{'itemscount'};
 }
 
-# }}}
 
-# {{{ sub Count
 
 =head2 Count
 
@@ -1451,9 +1373,7 @@ sub Count {
     }
 }
 
-# }}}
 
-# {{{ sub CountAll
 
 =head2 CountAll
 
@@ -1506,10 +1426,8 @@ sub CountAll {
     }
 }
 
-# }}}
 
 
-# {{{ sub IsLast
 
 =head2 IsLast
 
@@ -1530,9 +1448,7 @@ sub IsLast {
     }
 }
 
-# }}}
 
-# {{{ sub DEBUG
 
 sub DEBUG {
     my $self = shift;
@@ -1542,14 +1458,11 @@ sub DEBUG {
     return ( $self->{'DEBUG'} );
 }
 
-# }}}
 
 
 
 
-# }}}
 
-# {{{ Column
 
 =head2 Column { FIELD => undef } 
 
@@ -1603,9 +1516,7 @@ sub Column {
     return $column;
 }
 
-# }}}
 
-# {{{ Columns 
 
 
 =head2 Columns LIST
@@ -1619,12 +1530,10 @@ sub Columns {
     $self->Column( FIELD => $_ ) for @_;
 }
 
-# }}}
 
-# {{{ Fields
 
 =head2 Fields TABLE
- 
+
 Return a list of fields in TABLE, lowercased.
 
 TODO: Why are they lowercased?
@@ -1649,10 +1558,8 @@ sub Fields {
       };
 }
 
-# }}}
 
 
-# {{{ HasField
 
 =head2 HasField  { TABLE => undef, FIELD => undef }
 
@@ -1672,9 +1579,7 @@ sub HasField {
     return grep { $_ eq $field } $self->Fields($table);
 }
 
-# }}}
 
-# {{{ SetTable
 
 =head2 Table [TABLE]
 
@@ -1696,7 +1601,6 @@ sub Table {
 }
 
 
-# }}}
 if( eval { require capitalization } ) {
 	capitalization->unimport( __PACKAGE__ );
 }
@@ -1704,7 +1608,6 @@ if( eval { require capitalization } ) {
 1;
 __END__
 
-# {{{ POD
 
 
 =head1 TESTING
@@ -1738,7 +1641,6 @@ DBIx::SearchBuilder::Handle, DBIx::SearchBuilder::Record.
 
 =cut
 
-# }}}
 
 
 
