@@ -1,7 +1,7 @@
 # $Header: /home/jesse/DBIx-SearchBuilder/history/SearchBuilder/Handle.pm,v 1.21 2002/01/28 06:11:37 jesse Exp $
 package DBIx::SearchBuilder::Handle;
 use strict;
-use Carp;
+use Carp qw(croak cluck);
 use DBI;
 use Class::ReturnValue;
 use Encode;
@@ -13,7 +13,6 @@ $TRANSDEPTH = 0;
 $VERSION = '$Version$';
 
 
-# {{{ Top POD
 
 =head1 NAME
 
@@ -37,9 +36,7 @@ This class provides a wrapper for DBI handles that can also perform a number of
  
 =cut
 
-# }}}
 
-# {{{ sub new 
 
 =head2 new
 
@@ -57,9 +54,7 @@ sub new  {
     return $self;
 }
 
-# }}}
 
-# {{{ sub Connect 
 
 =head2 Connect PARAMHASH: Driver, Database, Host, User, Password
 
@@ -125,9 +120,7 @@ sub Connect  {
     return(undef);
 
 }
-# }}}
 
-# {{{ _UpgradeHandle
 
 =head2 _UpgradeHandle DRIVER
 
@@ -148,10 +141,8 @@ sub _UpgradeHandle {
     return 1;
 }
 
-# }}}
 
 
-# {{{ BuildDSN
 
 =head2 BuildDSN PARAMHASH
 
@@ -184,9 +175,7 @@ sub BuildDSN {
   $self->{'dsn'}= $dsn;
 }
 
-# }}}
 
-# {{{ DSN
 
 =head2 DSN
 
@@ -198,9 +187,7 @@ sub DSN {
     return($self->{'dsn'});
 }
 
-# }}}
 
-# {{{ RaiseError
 
 =head2 RaiseError [MODE]
 
@@ -218,9 +205,7 @@ sub RaiseError {
 }
 
 
-# }}}
 
-# {{{ PrintError
 
 =head2 PrintError [MODE]
 
@@ -238,7 +223,6 @@ sub PrintError {
 }
 
 
-# }}}
 
 =head2 LogSQLStatements BOOL
 
@@ -252,11 +236,10 @@ Returns whether we're currently logging or not as a boolean
 sub LogSQLStatements {
     my $self = shift;
     if (@_) {
-
         require Time::HiRes;
-    $self->{'_DoLogSQL'} = shift;
-    return ($self->{'_DoLogSQL'});
+        $self->{'_DoLogSQL'} = shift;
     }
+    return ($self->{'_DoLogSQL'});
 }
 
 =head2 _LogSQLStatement STATEMENT DURATION
@@ -269,7 +252,8 @@ sub _LogSQLStatement {
     my $self = shift;
     my $statement = shift;
     my $duration = shift;
-    push @{$self->{'StatementLog'}} , ([Time::Hires::time(), $statement, $duration]);
+    my @bind = @_;
+    push @{$self->{'StatementLog'}} , ([Time::HiRes::time(), $statement, [@bind], $duration]);
 
 }
 
@@ -301,7 +285,6 @@ sub SQLStatementLog {
 }
 
 
-# {{{ AutoCommit
 
 =head2 AutoCommit [MODE]
 
@@ -319,9 +302,7 @@ sub AutoCommit {
 }
 
 
-# }}}
 
-# {{{ sub Disconnect 
 
 =head2 Disconnect
 
@@ -338,9 +319,7 @@ sub Disconnect  {
   }
 }
 
-# }}}
 
-# {{{ sub Handle / dbh 
 
 =head2 dbh [HANDLE]
 
@@ -360,9 +339,7 @@ sub dbh {
   return($DBIHandle{$self} ||= $PrevHandle);
 }
 
-# }}}
 
-# {{{ sub Insert
 =head2 Insert $TABLE_NAME @KEY_VALUE_PAIRS
 
 Takes a table name and a set of key-value pairs in an array. splits the key value pairs, constructs an INSERT statement and performs the insert. Returns the row_id of this row.
@@ -389,9 +366,7 @@ sub Insert {
     my $sth =  $self->SimpleQuery($QueryString, @bind);
     return ($sth);
   }
-# }}}
 
-# {{{ sub UpdateRecordValue 
 
 =head2 UpdateRecordValue 
 
@@ -461,9 +436,7 @@ sub UpdateTableValue  {
 
     return $self->UpdateRecordValue(%args)
 }
-# }}}
 
-# {{{ sub SimpleQuery
 
 =head2 SimpleQuery QUERY_STRING, [ BIND_VALUE, ... ]
 
@@ -520,8 +493,7 @@ sub SimpleQuery {
         eval { $executed = $sth->execute(@bind_values) };
     }
     if ( $self->LogSQLStatements ) {
-        $self->_LogSQLStatement( $QueryString, tv_interval($basetime) );
-
+        $self->_LogSQLStatement( $QueryString, Time::HiRes::time() - $basetime, @bind_values );
     }
 
     if ( $@ or !$executed ) {
@@ -531,7 +503,7 @@ sub SimpleQuery {
 
         }
         else {
-            warn "$self couldn't execute the query '$QueryString'";
+            cluck "$self couldn't execute the query '$QueryString'";
 
             my $ret = Class::ReturnValue->new();
             $ret->as_error(
@@ -548,9 +520,7 @@ sub SimpleQuery {
 
 }
 
-# }}}
 
-# {{{ sub FetchResult
 
 =head2 FetchResult QUERY, [ BIND_VALUE, ... ]
 
@@ -573,9 +543,7 @@ sub FetchResult {
    return($sth);
   }
 }
-# }}}
 
-# {{{ BinarySafeBLOBs
 
 =head2 BinarySafeBLOBs
 
@@ -589,9 +557,7 @@ sub BinarySafeBLOBs {
     return(1);
 }
 
-# }}}
 
-# {{{ KnowsBLOBs
 
 =head2 KnowsBLOBs
 
@@ -605,9 +571,7 @@ sub KnowsBLOBs {
     return(1);
 }
 
-# }}}
 
-# {{{ BLOBParams
 
 =head2 BLOBParams FIELD_NAME FIELD_TYPE
 
@@ -622,9 +586,7 @@ sub BLOBParams {
     return ( {} );
 }
 
-# }}}
 
-# {{{ DatabaseVersion
 
 =head2 DatabaseVersion
 
@@ -642,9 +604,7 @@ sub DatabaseVersion {
         $self->{'database_version'}= $vals[0];
     }
 }
-# }}}
 
-# {{{ CaseSensitive
 
 =head2 CaseSensitive
 
@@ -659,10 +619,8 @@ sub CaseSensitive {
 }
 
 
-# }}} 
 
 
-# {{{ _MakeClauseCaseInsensitive
 
 =head2 _MakeClauseCaseInsensitive FIELD OPERATOR VALUE
 
@@ -687,9 +645,7 @@ sub _MakeClauseCaseInsensitive {
 }
 
 
-# }}}
 
-# {{{ BeginTransaction
 
 =head2 BeginTransaction
 
@@ -710,9 +666,7 @@ sub BeginTransaction {
     }
 }
 
-# }}}
 
-# {{{ Commit
 
 =head2 Commit
 
@@ -733,9 +687,7 @@ sub Commit {
     }
 }
 
-# }}}
 
-# {{{ Rollback
 
 =head2 Rollback [FORCE]
 
@@ -748,23 +700,28 @@ If this method is passed a true argument, stack depth is blown away and the oute
 
 sub Rollback {
     my $self = shift;
-    my $force = shift || undef;
-    #unless ($TRANSDEPTH) {Carp::confess("Attempted to rollback a transaction with none in progress")};
-    $TRANSDEPTH--;
+    my $force = shift;
 
-    if ($force) {
+    my $dbh = $self->dbh;
+    unless( $dbh ) {
         $TRANSDEPTH = 0;
-       return($self->dbh->rollback);
+        return;
     }
 
+    #unless ($TRANSDEPTH) {Carp::confess("Attempted to rollback a transaction with none in progress")};
+    if ($force) {
+        $TRANSDEPTH = 0;
+        return($dbh->rollback);
+    }
+
+    $TRANSDEPTH-- if ($TRANSDEPTH >= 1);
     if ($TRANSDEPTH == 0 ) {
-       return($self->dbh->rollback);
+        return($dbh->rollback);
     } else { #we're inside a transaction
         return($TRANSDEPTH);
     }
 }
 
-# }}}
 
 =head2 ForceRollback
 
@@ -790,7 +747,6 @@ sub TransactionDepth {
 }
 
 
-# {{{ ApplyLimits
 
 =head2 ApplyLimits STATEMENTREF ROWS_PER_PAGE FIRST_ROW
 
@@ -820,10 +776,8 @@ sub ApplyLimits {
 }
 
 
-# }}}
 
 
-# {{{ Join
 
 =head2 Join { Paramhash }
 
@@ -904,7 +858,6 @@ sub Join {
         if ( !$alias || $args{'ALIAS1'} ) {
             return ( $self->_NormalJoin(%args) );
         }
-
         $args{'SearchBuilder'}->{'aliases'} = \@new_aliases;
     }
 
@@ -960,7 +913,6 @@ sub _NormalJoin {
 
     if ( $args{'TYPE'} =~ /LEFT/i ) {
         my $alias = $sb->_GetAlias( $args{'TABLE2'} );
-
         $sb->{'left_joins'}{"$alias"}{'alias_string'} =
           " LEFT JOIN $args{'TABLE2'} $alias ";
 
@@ -1027,9 +979,7 @@ sub _BuildJoins {
 
 }
 
-# }}}
 
-# {{{ DistinctQuery
 
 =head2 DistinctQuery STATEMENTREF 
 
@@ -1041,17 +991,16 @@ takes an incomplete SQL SELECT statement and massages it to return a DISTINCT re
 sub DistinctQuery {
     my $self = shift;
     my $statementref = shift;
-    #my $table = shift;
+    my $sb = shift;
 
     # Prepend select query for DBs which allow DISTINCT on all column types.
     $$statementref = "SELECT DISTINCT main.* FROM $$statementref";
-
+    $$statementref .= $sb->_GroupClause;
+    $$statementref .= $sb->_OrderClause;
 }
 
-# }}}
 
 
-# {{{ DistinctCount
 
 =head2 DistinctCount STATEMENTREF 
 
@@ -1069,7 +1018,6 @@ sub DistinctCount {
 
 }
 
-# }}}
 
 =head2 Log MESSAGE
 
@@ -1105,7 +1053,6 @@ sub DESTROY {
 1;
 __END__
 
-# {{{ POD
 
 =head1 AUTHOR
 
@@ -1117,4 +1064,3 @@ perl(1), L<DBIx::SearchBuilder>
 
 =cut
 
-# }}}
